==> tabular-data-template-1.0.1-20140324.155213-8-servicearchive.tar.gz: org/gcube/portlets/user/tdtemplate/client/template/view/TemplatePanel.java <==
/**
 * 
 */
package org.gcube.portlets.user.tdtemplate.client.template.view;

import java.util.ArrayList;
import java.util.List;

import org.gcube.portlets.user.tdtemplate.client.TdTemplateConstants;
import org.gcube.portlets.user.tdtemplate.client.TdTemplateController;
import org.gcube.portlets.user.tdtemplate.client.event.TemplateComplitedEvent;
import org.gcube.portlets.user.tdtemplate.client.resources.TdTemplateAbstractResources;
import org.gcube.portlets.user.tdtemplate.client.template.SetColumnTypeDefinition;
import org.gcube.portlets.user.tdtemplate.client.template.SetColumnTypeDialogManager;

import com.extjs.gxt.ui.client.Style.ButtonScale;
import com.extjs.gxt.ui.client.Style.IconAlign;
import com.extjs.gxt.ui.client.Style.Scroll;
import com.extjs.gxt.ui.client.event.BaseEvent;
import com.extjs.gxt.ui.client.event.ButtonEvent;
import com.extjs.gxt.ui.client.event.Events;
import com.extjs.gxt.ui.client.event.Listener;
import com.extjs.gxt.ui.client.event.SelectionListener;
import com.extjs.gxt.ui.client.widget.ContentPanel;
import com.extjs.gxt.ui.client.widget.LayoutContainer;
import com.extjs.gxt.ui.client.widget.VerticalPanel;
import com.extjs.gxt.ui.client.widget.button.Button;
import com.extjs.gxt.ui.client.widget.button.ButtonGroup;
import com.extjs.gxt.ui.client.widget.menu.SeparatorMenuItem;
import com.extjs.gxt.ui.client.widget.toolbar.ToolBar;
import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.ui.AbstractImagePrototype;
import com.google.gwt.user.client.ui.FlexTable;
import com.google.gwt.user.client.ui.HTMLTable;
import com.google.gwt.user.client.ui.HasHorizontalAlignment;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.Widget;

/**
 * @author Francesco Mangiacrapa francesco.mangiacrapa@isti.cnr.it
 * @Jan 8, 2014
 *
 */
public class TemplatePanel {
	

	/**
	 * An instance of the constants.
	 */
	private FlexTable flexTableTemplate = new FlexTable();
	private int numColumns;
	private ContentPanel tableContainer = new ContentPanel();
	private VerticalPanel verticalPanel = new VerticalPanel();
	private ContentPanel suggestionContainer = new ContentPanel();
	private LayoutContainer principalContainer = new LayoutContainer();
	
	private List<ColumnDefinitionView> columnsDefined;
	private TemplateSwitcherInteface templateSwitcherInteface;
	private SetColumnTypeDefinition setColumnTypeDefinition;
	
	/**
	 * @param setColumnTypeDefinition 
	 * 
	 */
	public TemplatePanel(TemplateSwitcherInteface templateSwitcherInteface) {
		this.numColumns = templateSwitcherInteface.getNumberOfColumns();
		this.templateSwitcherInteface = templateSwitcherInteface;
		this.columnsDefined = new ArrayList<ColumnDefinitionView>(numColumns);
		

		setColumnTypeDefinition = new SetColumnTypeDefinition(templateSwitcherInteface) {
			
			@Override
			public void updateListCategory() {
				
//				setColumnTypeViewManager = new SetColumnTypeDialogManager(setColumnTypeDefinition.getListCategory());
				tableContainer.setEnabled(true);
				initTableColumns(flexTableTemplate, 0, numColumns);
			}
		};
		

		this.tableContainer.setHeaderVisible(false);
		this.tableContainer.setBorders(false);
//		this.tableContainer.setLayout(new FitLayout());
//		this.tableContainer.setStyleAttribute("margin-top", "50px");
		inizializeTableTemplate();
		inizializeOperationsPanel();
	
		this.tableContainer.setScrollMode(Scroll.AUTOX);
		this.tableContainer.setEnabled(false);
		
//		suggestionContainer.setStyleAttribute("margin-top", "5px");
		suggestionContainer.setStyleAttribute("margin-bottom", "10px");
		suggestionContainer.setScrollMode(Scroll.AUTOY);
		suggestionContainer.setHeaderVisible(false);
		suggestionContainer.setBorders(false);
		suggestionContainer.setBodyBorder(true);
		
		addNewSuggestion(TdTemplateConstants.SUGGESTION, "Please, set type to all columns");
		
		
//		suggestionContainer.getElement().getStyle().setBorderColor("#32CD32");

		tableContainer.add(suggestionContainer);
		tableContainer.add(flexTableTemplate);
		

		principalContainer.add(tableContainer);

		suggestionContainer.addListener(Events.Render, new Listener<BaseEvent>() {

			@Override
			public void handleEvent(BaseEvent be) {
				setBorderAsOnError(false);
				
			}
		});
		
	}
	
	public void setBorderAsOnError(boolean bool){
		
		if(suggestionContainer.isRendered() && suggestionContainer.getElement("body")!=null){
			if(bool)
				suggestionContainer.getElement("body").getStyle().setBorderColor("#FF2300");
			else
				suggestionContainer.getElement("body").getStyle().setBorderColor("#99BBE8");
		}
	}
	
	public void addNewSuggestion(String title, String text){
		SuggestionLabel sugg = new SuggestionLabel(title, text, templateSwitcherInteface.getTdTTemplateType().getConstraintDescription());
		addSuggestionLabel(sugg);
	}
	
	public void addNewSuggestion(String title, String text, AbstractImagePrototype img){
		SuggestionLabel sugg = new SuggestionLabel(title, text,  templateSwitcherInteface.getTdTTemplateType().getConstraintDescription(), img);
		addSuggestionLabel(sugg);
	}
	
	private void addSuggestionLabel(SuggestionLabel sugg){
		suggestionContainer.removeAll();
		suggestionContainer.add(sugg.getHtml());
		tableContainer.layout();
	}


	/**
	 * 
	 */
	private void inizializeOperationsPanel() {
		
		ToolBar toolbar = new ToolBar();
		
		ButtonGroup group = new ButtonGroup(1);  
	    group.setHeading("Modify Column");  


		// Add a button that will add more rows to the table
	    Button addColumnButton = new Button("Add Column", TdTemplateAbstractResources.columnAdd());
		addColumnButton.setScale(ButtonScale.SMALL);
		addColumnButton.setIconAlign(IconAlign.TOP);
//		addColumnButton.setArrowAlign(ButtonArrowAlign.BOTTOM);
		
		addColumnButton.addSelectionListener(new SelectionListener<ButtonEvent>() {
			
			@Override
			public void componentSelected(ButtonEvent ce) {
				addColumn(flexTableTemplate);
				
			}
		});

		Button removeRowButton = new Button("Remove Column", TdTemplateAbstractResources.columnRemove());
		removeRowButton.setScale(ButtonScale.SMALL);
		removeRowButton.setIconAlign(IconAlign.TOP);
//		removeRowButton.setArrowAlign(ButtonArrowAlign.BOTTOM);
		
		removeRowButton.addSelectionListener(new SelectionListener<ButtonEvent>() {

			@Override
			public void componentSelected(ButtonEvent ce) {
				if(numColumns>1){
					removeColumn(flexTableTemplate, numColumns-1);
					setNumColumns(numColumns-1);
				}
				
			}
		});
		

		toolbar.add(addColumnButton);
		toolbar.add(new SeparatorMenuItem());
		toolbar.add(removeRowButton);
		tableContainer.setTopComponent(toolbar);
	}


	/**
	 * Initialize this example.
	 */

	public void inizializeTableTemplate() {
		// Create a Flex Table
		flexTableTemplate.addStyleName("FlexTableTemplate");
//		initTableColumns(flexTableTemplate, 0, numColumns);
		
		flexTableTemplate.ensureDebugId("cwFlexTable");
	}

	/**
	 * Add a row to the flex table.
	 */
	private void addRow(FlexTable flexTable) {
		int numRows = flexTable.getRowCount();
//		flexTable.setWidget(numRows, 0, new Image(ResourcesTemplate.INSTANCE.getArrowDown()));
//		flexTable.setWidget(numRows, 1, new Image(ResourcesTemplate.INSTANCE.getArrowDown()));
//		flexTable.getFlexCellFormatter().setRowSpan(0, 1, numRows + 1);
	}
	
	
	private void initTableColumns(FlexTable flexTable, int columnIndex, int columnsOffset){
		
		for (int i = 0; i < columnsOffset; i++) {
			
			int indexOffset = columnIndex+i;
			
			ColumnDefinitionView columnDef = new ColumnDefinitionView(this, indexOffset, new SetColumnTypeDialogManager(setColumnTypeDefinition.getListCategory()));

			HorizontalPanel hp = new HorizontalPanel();
			hp.setHorizontalAlignment(HasHorizontalAlignment.ALIGN_CENTER);
//			hp.add(columnDef.getButtonType());
//			hp.add(columnDef.getButtonRule());
			
//			columnDef.getSetColumnTypeViewManager().getDialog();
			hp.add(columnDef.getSetColumnTypeViewManager().getTypingContainer());
//			hp.add(columnDef.getButtonRule());
			

			flexTable.setWidget(0, indexOffset, columnDef.getColumnHeaderValue());
			flexTable.setWidget(1, indexOffset, hp);
//			flexTable.getCellFormatter().getElement(1, columnIndex+i).setAttribute("text-align", "center");
			
			flexTable.setWidget(2, indexOffset, columnDef.getColumnDescription());
			flexTable.getCellFormatter().getElement(2, indexOffset).setAttribute("height", "40px");
			
			
			HTMLTable.RowFormatter rf = flexTable.getRowFormatter();
			rf.addStyleName(0, "FlexTableTemplate-header-row");
			rf.addStyleName(1, "FlexTableTemplate-other-rows");
			rf.addStyleName(2, "FlexTableTemplate-other-rows");

//			//TODO
//			int size = columnsDefined.size();
//			if(size>0 && indexOffset<size && columnsDefined.get(indexOffset)!=null)
//				columnsDefined.set(indexOffset, columnDef);
//			else
//				columnsDefined.add(indexOffset, columnDef);
			
			try{
				GWT.log("Setting indexOffset: "+indexOffset);
				columnsDefined.set(indexOffset, columnDef);
				
			}catch (Exception e) {
				GWT.log("Warn: indexOffset "+indexOffset +" doesn't exists into columnsDefined adding");
		
				columnsDefined.add(indexOffset, columnDef);
			}
		}
		
	}
	
	public void setWidgetIntoTable(int rowIndex, int columnIndex, Widget widget){
		HTMLTable.RowFormatter rf = flexTableTemplate.getRowFormatter();
		
		if(rowIndex>0)
			rf.addStyleName(rowIndex, "FlexTableTemplate-other-rows");
		
		flexTableTemplate.setWidget(rowIndex, columnIndex, widget);
	}
	
	public void clearCell(int rowIndex, int columnIndex){
		
		try{
			GWT.log("Clear cell rowIndex, "+rowIndex+", column index "+columnIndex);
			flexTableTemplate.clearCell(rowIndex, columnIndex);	
		}catch (Exception e) {
			GWT.log("Last remove cell throw exception");
		}
	}
	
	
	/**
	 * Add a row to the flex table.
	 */
	private void addColumn(FlexTable flexTable) {
		
//		int numRows = flexTable.getRowCount();
//		int numColumns = flexTable.getCellCount(0);

		initTableColumns(flexTable, numColumns, 1);
		
		setNumColumns(numColumns+1);
		
		validateTemplate();
		
//		flexTable.getFlexCellFormatter().setColSpan(0, 1, numRows + 1);
	}
	

	/**
	 * @param numColumns
	 */
	private void setNumColumns(int numColumns) {
		this.numColumns = numColumns;
		
	}


	private void addHeaderColumns(FlexTable flexTable) {
		int numRows = flexTable.getRowCount();
		
		int numColumns = flexTable.getCellCount(numRows);
		
		for (int i = 0; i < numColumns; i++) {
			flexTable.setWidget(1, i, new Label("Column "+i+1));
		}
	}
	
	/**
	 * Remove a row from the flex table.
	 */
	private void removeColumn(FlexTable flexTable, int columnIndex) {

	
		GWT.log("Remove column at index: "+columnIndex);
		
		if (columnIndex > 0) {
			int numRows = flexTable.getRowCount();
			GWT.log("Current Table row size: "+numRows);
			try{
				for (int i=0; i<numRows; i++) {
					GWT.log("RemoveCell i, "+i+", column index "+columnIndex);
					flexTable.removeCell(i, columnIndex);
				}
			}catch (Exception e) {
				GWT.log("Last remove cell throw exception");
			}
	
			GWT.log("Remove column defined at index: "+columnIndex);
			columnsDefined.remove(columnIndex);
			validateTemplate();
//			flexTable.getFlexCellFormatter().setRowSpan(0, 1, columnIndex - 1);
		}
	}

	/**
	 * Remove a row from the flex table.
	 */
	private void removeRow(FlexTable flexTable) {
		int numRows = flexTable.getRowCount();
		if (numRows > 1) {
			flexTable.removeRow(numRows - 1);
			flexTable.getFlexCellFormatter().setRowSpan(0, 1, numRows - 1);
		}
	}


	public int getNumColumns() {
		return numColumns;
	}


	public LayoutContainer getPanel() {
		return principalContainer;
	}


	public List<ColumnDefinitionView> getColumnsDefined() {
		return columnsDefined;
	}

	/**
	 * 
	 */
	public void validateTemplate() {
		
		boolean isValid = true;
		for (ColumnDefinitionView col : columnsDefined) {
			if(!col.isValid()){
				isValid = false;
				addNewSuggestion(TdTemplateConstants.SUGGESTION, TdTemplateConstants.PLEASE_SET_TYPE_TO_COLUMN_NUMBER+(col.getColumnIndex()+1));
				TdTemplateController.getInternalBus().fireEvent(new TemplateComplitedEvent(false));
				break;
			}
		}
		
		if(isValid){
			addNewSuggestion(TdTemplateConstants.TEMPLATE_COMPLETED, TdTemplateConstants.DO_YOU_WANT_SAVE_THE_TEMPLATE, TdTemplateAbstractResources.handsUP());
			TdTemplateController.getInternalBus().fireEvent(new TemplateComplitedEvent(true));
		}
		
	}

}

==> tabular-data-template-1.1.0-20140410.122037-19-servicearchive.tar.gz: org/gcube/portlets/user/tdtemplate/client/template/view/TemplatePanel.java <==
/**
 * 
 */
package org.gcube.portlets.user.tdtemplate.client.template.view;

import java.util.ArrayList;
import java.util.List;

import org.gcube.portlets.user.tdtemplate.client.TdTemplateConstants;
import org.gcube.portlets.user.tdtemplate.client.TdTemplateController;
import org.gcube.portlets.user.tdtemplate.client.event.SetColumnTypeCompletedEvent;
import org.gcube.portlets.user.tdtemplate.client.event.TemplateComplitedEvent;
import org.gcube.portlets.user.tdtemplate.client.resources.TdTemplateAbstractResources;
import org.gcube.portlets.user.tdtemplate.client.template.SetColumnTypeDefinition;
import org.gcube.portlets.user.tdtemplate.client.template.SetColumnTypeDialogManager;

import com.extjs.gxt.ui.client.Style.ButtonScale;
import com.extjs.gxt.ui.client.Style.IconAlign;
import com.extjs.gxt.ui.client.Style.Scroll;
import com.extjs.gxt.ui.client.event.BaseEvent;
import com.extjs.gxt.ui.client.event.ButtonEvent;
import com.extjs.gxt.ui.client.event.Events;
import com.extjs.gxt.ui.client.event.Listener;
import com.extjs.gxt.ui.client.event.SelectionListener;
import com.extjs.gxt.ui.client.widget.ContentPanel;
import com.extjs.gxt.ui.client.widget.LayoutContainer;
import com.extjs.gxt.ui.client.widget.button.Button;
import com.extjs.gxt.ui.client.widget.button.ButtonGroup;
import com.extjs.gxt.ui.client.widget.form.CheckBox;
import com.extjs.gxt.ui.client.widget.menu.SeparatorMenuItem;
import com.extjs.gxt.ui.client.widget.toolbar.ToolBar;
import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.ui.AbstractImagePrototype;
import com.google.gwt.user.client.ui.FlexTable;
import com.google.gwt.user.client.ui.HTMLTable;
import com.google.gwt.user.client.ui.HasHorizontalAlignment;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.Widget;

/**
 * @author Francesco Mangiacrapa francesco.mangiacrapa@isti.cnr.it
 * @Jan 8, 2014
 *
 */
public class TemplatePanel {
	

	/**
	 * 
	 */
	
	/**
	 * An instance of the constants.
	 */
	protected FlexTable flexTableTemplate = new FlexTable();
	private int numColumns;
	protected ContentPanel tableContainer = new ContentPanel();
	private ContentPanel suggestionContainer = new ContentPanel();
	private LayoutContainer suggestionDescription = new LayoutContainer();
	private LayoutContainer principalContainer = new LayoutContainer();
	
	protected List<ColumnDefinitionView> columnsDefined;
	private TemplateSwitcherInteface templateSwitcherInteface;
	protected SetColumnTypeDefinition setColumnTypeDefinition;
	private TdTemplateController controller;
	
	
	/**
	 * @param setColumnTypeDefinition 
	 * 
	 */
	public TemplatePanel(TemplateSwitcherInteface templateSwitcherInteface, TdTemplateController controller) {
		this.numColumns = templateSwitcherInteface.getNumberOfColumns();
		this.controller = controller;
		this.templateSwitcherInteface = templateSwitcherInteface;
		this.columnsDefined = new ArrayList<ColumnDefinitionView>(numColumns);
	
		setColumnTypeDefinition = new SetColumnTypeDefinition(templateSwitcherInteface) {
			
			@Override
			public void updateListCategory() {
				GWT.log("Init table");
				tableContainer.setEnabled(true);
				initTableColumns(flexTableTemplate, 0, numColumns);
				TdTemplateController.getInternalBus().fireEvent(new SetColumnTypeCompletedEvent());
			}
		};
		

		this.tableContainer.setHeaderVisible(false);
		this.tableContainer.setBorders(false);
//		this.tableContainer.setLayout(new FitLayout());
//		this.tableContainer.setStyleAttribute("margin-top", "50px");
		inizializeTableTemplate();
		inizializeOperationsPanel();
	
		this.tableContainer.setScrollMode(Scroll.AUTOX);
		this.tableContainer.setEnabled(false);
		
//		suggestionContainer.setStyleAttribute("margin-top", "5px");
//		suggestionContainer.setStyleAttribute("margin-bottom", "10px");
		suggestionContainer.setScrollMode(Scroll.AUTOY);
		suggestionContainer.setHeaderVisible(false);
		suggestionContainer.setBorders(false);
		suggestionContainer.setBodyBorder(true);
		
		
		suggestionContainer.add(suggestionDescription);

//		suggestionContainer.add(constraintSuggestion.getHtml());
		
		addNewSuggestion(TdTemplateConstants.SUGGESTION, "Please, set type to all columns");
//		suggestionContainer.getElement().getStyle().setBorderColor("#32CD32");

		tableContainer.add(suggestionContainer);
		tableContainer.add(flexTableTemplate);
		
		
		String title = templateSwitcherInteface.getType() + " columns constraints";
		ConstraintSuggestionLabel cs = new ConstraintSuggestionLabel(title, templateSwitcherInteface.getTdTTemplateType().getConstraintDescription(), false);
//		tableContainer.add(cs.getAnchor());

		principalContainer.add(tableContainer);
		principalContainer.add(cs);


		suggestionContainer.addListener(Events.Render, new Listener<BaseEvent>() {

			@Override
			public void handleEvent(BaseEvent be) {
				setBorderAsOnError(false);
				
			}
		});
		
	}
	

	
	
	public void setBorderAsOnError(boolean bool){
		
		if(suggestionContainer.isRendered() && suggestionContainer.getElement("body")!=null){
			if(bool)
				suggestionContainer.getElement("body").getStyle().setBorderColor("#FF2300");
			else
				suggestionContainer.getElement("body").getStyle().setBorderColor("#99BBE8");
		}
	}
	
	public void addNewSuggestion(String title, String text){
		SuggestionLabel sugg = new SuggestionLabel(title, text, "");
		addSuggestionLabel(sugg);
	}
	
	public void addNewSuggestion(String title, String text, AbstractImagePrototype img){
		SuggestionLabel sugg = new SuggestionLabel(title, text,  "", img);
		addSuggestionLabel(sugg);
	}
	
	private void addSuggestionLabel(SuggestionLabel sugg){
		suggestionDescription.removeAll();
		suggestionDescription.add(sugg.getHtml());
		tableContainer.layout();
	}


	/**
	 * 
	 */
	private void inizializeOperationsPanel() {
		
		ToolBar toolbar = new ToolBar();
		
		ButtonGroup group = new ButtonGroup(1);  
	    group.setHeading("Modify Column");  


		// Add a button that will add more rows to the table
	    Button addColumnButton = new Button("Add Column", TdTemplateAbstractResources.columnAdd());
		addColumnButton.setScale(ButtonScale.MEDIUM);
		addColumnButton.setIconAlign(IconAlign.TOP);
//		addColumnButton.setArrowAlign(ButtonArrowAlign.BOTTOM);
		
		addColumnButton.addSelectionListener(new SelectionListener<ButtonEvent>() {
			
			@Override
			public void componentSelected(ButtonEvent ce) {
				addColumn(flexTableTemplate);
				
			}
		});

		Button removeRowButton = new Button("Remove Column", TdTemplateAbstractResources.columnRemove());
		removeRowButton.setScale(ButtonScale.MEDIUM);
		removeRowButton.setIconAlign(IconAlign.TOP);
//		removeRowButton.setArrowAlign(ButtonArrowAlign.BOTTOM);
		
		removeRowButton.addSelectionListener(new SelectionListener<ButtonEvent>() {

			@Override
			public void componentSelected(ButtonEvent ce) {
				if(numColumns>1){
					removeColumn(flexTableTemplate, numColumns-1);
					setNumColumns(numColumns-1);
				}
				
			}
		});
		

		toolbar.add(addColumnButton);
		toolbar.add(new SeparatorMenuItem());
		toolbar.add(removeRowButton);
		tableContainer.setTopComponent(toolbar);
	}


	/**
	 * Initialize this example.
	 */

	public void inizializeTableTemplate() {
		// Create a Flex Table
		flexTableTemplate.addStyleName("FlexTableTemplate");
//		initTableColumns(flexTableTemplate, 0, numColumns);
		
		flexTableTemplate.ensureDebugId("cwFlexTable");
	}

	/**
	 * Add a row to the flex table.
	 */
	private void addRow(FlexTable flexTable) {
		int numRows = flexTable.getRowCount();
//		flexTable.setWidget(numRows, 0, new Image(ResourcesTemplate.INSTANCE.getArrowDown()));
//		flexTable.setWidget(numRows, 1, new Image(ResourcesTemplate.INSTANCE.getArrowDown()));
//		flexTable.getFlexCellFormatter().setRowSpan(0, 1, numRows + 1);
	}
	
	
	private void initTableColumns(FlexTable flexTable, int columnIndex, int columnsOffset){
		
		for (int i = 0; i < columnsOffset; i++) {
			
			int indexOffset = columnIndex+i;
			
			ColumnDefinitionView columnDef = new ColumnDefinitionView(this, indexOffset, new SetColumnTypeDialogManager(setColumnTypeDefinition.getListCategory(), controller));

			HorizontalPanel hp = new HorizontalPanel();
			hp.setHorizontalAlignment(HasHorizontalAlignment.ALIGN_CENTER);

			hp.add(columnDef.getSetColumnTypeViewManager().getTypingContainer());

			CheckBox check = new CheckBox();
			check.setStyleAttribute("margin-right", "10px");
			
			/*HorizontalPanel hp2 = new HorizontalPanel();
			hp2.setVerticalAlignment(HasVerticalAlignment.ALIGN_MIDDLE);
			hp2.add(check);
			hp2.add(columnDef.getColumnHeaderValue());
			flexTable.setWidget(0, indexOffset, hp2);*/
			
			flexTable.setWidget(0, indexOffset, columnDef.getColumnHeaderValue());
			flexTable.setWidget(1, indexOffset, hp);
			
			flexTable.setWidget(2, indexOffset, columnDef.getColumnDescription());
			flexTable.getCellFormatter().getElement(2, indexOffset).setAttribute("height", "40px");
			
			
			HTMLTable.RowFormatter rf = flexTable.getRowFormatter();
			rf.addStyleName(0, "FlexTableTemplate-header-row");
			rf.addStyleName(1, "FlexTableTemplate-other-rows");
			rf.addStyleName(2, "FlexTableTemplate-other-rows");

//			//TODO
//			int size = columnsDefined.size();
//			if(size>0 && indexOffset<size && columnsDefined.get(indexOffset)!=null)
//				columnsDefined.set(indexOffset, columnDef);
//			else
//				columnsDefined.add(indexOffset, columnDef);
			
			try{
				GWT.log("Setting indexOffset: "+indexOffset);
				columnsDefined.set(indexOffset, columnDef);
				
			}catch (Exception e) {
				GWT.log("Warn: indexOffset "+indexOffset +" doesn't exists into columnsDefined adding");
		
				columnsDefined.add(indexOffset, columnDef);
			}
		}
		
	}
	
	public void setWidgetIntoTable(int rowIndex, int columnIndex, Widget widget){
		HTMLTable.RowFormatter rf = flexTableTemplate.getRowFormatter();
		
		if(rowIndex>0)
			rf.addStyleName(rowIndex, "FlexTableTemplate-other-rows");
		
		flexTableTemplate.setWidget(rowIndex, columnIndex, widget);
	}
	
	public void clearCell(int rowIndex, int columnIndex){
		
		try{
			GWT.log("Clear cell rowIndex, "+rowIndex+", column index "+columnIndex);
			flexTableTemplate.clearCell(rowIndex, columnIndex);	
		}catch (Exception e) {
			GWT.log("Last remove cell throw exception");
		}
	}
	
	
	/**
	 * Add a row to the flex table.
	 */
	private void addColumn(FlexTable flexTable) {
		
//		int numRows = flexTable.getRowCount();
//		int numColumns = flexTable.getCellCount(0);

		initTableColumns(flexTable, numColumns, 1);
		
		setNumColumns(numColumns+1);
		
		validateTemplate();
		
//		flexTable.getFlexCellFormatter().setColSpan(0, 1, numRows + 1);
	}
	

	/**
	 * @param numColumns
	 */
	private void setNumColumns(int numColumns) {
		this.numColumns = numColumns;
		
	}


	private void addHeaderColumns(FlexTable flexTable) {
		int numRows = flexTable.getRowCount();
		
		int numColumns = flexTable.getCellCount(numRows);
		
		for (int i = 0; i < numColumns; i++) {
			flexTable.setWidget(1, i, new Label("Column "+i+1));
		}
	}
	
	/**
	 * Remove a row from the flex table.
	 */
	private void removeColumn(FlexTable flexTable, int columnIndex) {

	
		GWT.log("Remove column at index: "+columnIndex);
		
		if (columnIndex > 0) {
			int numRows = flexTable.getRowCount();
			GWT.log("Current Table row size: "+numRows);
			try{
				for (int i=0; i<numRows; i++) {
					GWT.log("RemoveCell i, "+i+", column index "+columnIndex);
					flexTable.removeCell(i, columnIndex);
				}
			}catch (Exception e) {
				GWT.log("Last remove cell throw exception");
			}
	
			GWT.log("Remove column defined at index: "+columnIndex);
			columnsDefined.remove(columnIndex);
			validateTemplate();
//			flexTable.getFlexCellFormatter().setRowSpan(0, 1, columnIndex - 1);
		}
	}

	/**
	 * Remove a row from the flex table.
	 */
	private void removeRow(FlexTable flexTable) {
		int numRows = flexTable.getRowCount();
		if (numRows > 1) {
			flexTable.removeRow(numRows - 1);
			flexTable.getFlexCellFormatter().setRowSpan(0, 1, numRows - 1);
		}
	}


	public int getNumColumns() {
		return numColumns;
	}


	public LayoutContainer getPanel() {
		return principalContainer;
	}


	public List<ColumnDefinitionView> getColumnsDefined() {
		return columnsDefined;
	}

	/**
	 * 
	 */
	public void validateTemplate() {
		
		boolean isValid = true;
		for (ColumnDefinitionView col : columnsDefined) {
			if(!col.isValid()){
				isValid = false;
				addNewSuggestion(TdTemplateConstants.SUGGESTION, TdTemplateConstants.PLEASE_SET_TYPE_TO_COLUMN_NUMBER+(col.getColumnIndex()+1));
				TdTemplateController.getInternalBus().fireEvent(new TemplateComplitedEvent(false));
				break;
			}
		}
		
		if(isValid){
			addNewSuggestion(TdTemplateConstants.TEMPLATE_COMPLETED, TdTemplateConstants.NOW_IS_POSSIBLE_TO_SAVE_THE_TEMPLATE_CREATED, TdTemplateAbstractResources.handsUP());
			TdTemplateController.getInternalBus().fireEvent(new TemplateComplitedEvent(true));
		}
	}
}
